@@ -229,10 +229,13 @@ def get_kernel_ramdisk_info(node_uuid, driver_info, mode='deploy',
     image_info = {}
     labels = KERNEL_RAMDISK_LABELS[mode]
     for label in labels:
-        image_info[label] = (
-            str(driver_info[label]),
-            os.path.join(root_dir, node_uuid, label)
-        )
+        try:
+            image_info[label] = (
+                str(driver_info[label]),
+                os.path.join(root_dir, node_uuid, label)
+            )
+        except KeyError:
+            pass  # may be absent in rare cases, verified in parse_driver_info
     return image_info
 
 
@@ -599,8 +602,7 @@ def is_ipxe_enabled(task):
     """Return true if ipxe is set.
 
     :param task: A TaskManager object
-    :returns: boolean true if ``[pxe]ipxe_enabled`` is configured
-              or if the task driver instance is the iPXE driver.
+    :returns: boolean true if the task driver instance is the iPXE driver.
     """
     return 'ipxe_boot' in task.driver.boot.capabilities
 
@@ -620,6 +622,11 @@ def parse_driver_info(node, mode='deploy'):
     :returns: A dict with the driver_info values.
     :raises: MissingParameterValue
     """
+    if not deploy_utils.needs_agent_ramdisk(node, mode=mode):
+        # Ramdisk deploy does not need an agent, nor does it support any other,
+        # options. Skipping.
+        return {}
+
     info = node.driver_info
 
     params_to_check = KERNEL_RAMDISK_LABELS[mode]
@@ -667,41 +674,60 @@ def get_instance_image_info(task, ipxe_enabled=False):
             os.path.join(root_dir, node.uuid, 'boot_iso'))
 
         return image_info
-
     image_properties = None
     d_info = deploy_utils.get_image_instance_info(node)
+    isap = node.driver_internal_info.get('is_source_a_path')
 
     def _get_image_properties():
-        nonlocal image_properties
-        if not image_properties:
-            glance_service = service.GlanceImageService(context=ctx)
-            image_properties = glance_service.show(
+        nonlocal image_properties, isap
+        if not image_properties and not isap:
+            i_service = service.get_image_service(
+                d_info['image_source'],
+                context=ctx)
+            image_properties = i_service.show(
                 d_info['image_source'])['properties']
+        # TODO(TheJulia): At some point, we should teach this code
+        # to understand that with a path, it *can* retrieve the
+        # manifest from the HTTP(S) endpoint, which can populate
+        # image_properties, and drive path to variable population
+        # like is done with basically Glance.
 
     labels = ('kernel', 'ramdisk')
+    if not isap:
+        anaconda_labels = ('stage2', 'ks_template', 'ks_cfg')
+    else:
+        # When a path is used, a stage2 ramdisk can be determiend
+        # automatically by anaconda, so it is not an explicit
+        # requirement.
+        anaconda_labels = ('ks_template', 'ks_cfg')
+
     if not (i_info.get('kernel') and i_info.get('ramdisk')):
         # NOTE(rloo): If both are not specified in instance_info
         # we won't use any of them. We'll use the values specified
         # with the image, which we assume have been set.
         _get_image_properties()
-        for label in labels:
-            i_info[label] = str(image_properties[label + '_id'])
-        node.instance_info = i_info
-        node.save()
+        if image_properties:
+            # This is intended for Glance usage, but all image properties
+            # should be routed through the image service request routing.
+            for label in labels:
+                i_info[label] = str(image_properties[label + '_id'])
+            node.instance_info = i_info
+            node.save()
+        # TODO(TheJulia): Add functionality to look/grab the hints file
+        # for anaconda and just run with the entire path.
 
-    anaconda_labels = ()
-    if deploy_utils.get_boot_option(node) == 'kickstart':
         # stage2: installer stage2 squashfs image
         # ks_template: anaconda kickstart template
         # ks_cfg - rendered ks_template
-        anaconda_labels = ('stage2', 'ks_template', 'ks_cfg')
+
         # NOTE(rloo): We save stage2 & ks_template values in case they
         # are changed by the user after we start using them and to
         # prevent re-computing them again.
         if not node.driver_internal_info.get('stage2'):
             if i_info.get('stage2'):
                 node.set_driver_internal_info('stage2', i_info['stage2'])
-            else:
+            elif not isap:
+                # If the source is not a path, then we need a stage2 ramdisk.
                 _get_image_properties()
                 if 'stage2_id' not in image_properties:
                     msg = (_("'stage2_id' is missing from the properties of "
@@ -713,19 +739,32 @@ def get_instance_image_info(task, ipxe_enabled=False):
                 else:
                     node.set_driver_internal_info(
                         'stage2', str(image_properties['stage2_id']))
-            if i_info.get('ks_template'):
-                node.set_driver_internal_info('ks_template',
-                                              i_info['ks_template'])
-            else:
-                _get_image_properties()
-                # ks_template is an optional property on the image
-                if 'ks_template' not in image_properties:
-                    node.set_driver_internal_info(
-                        'ks_template', CONF.anaconda.default_ks_template)
-                else:
-                    node.set_driver_internal_info(
-                        'ks_template', str(image_properties['ks_template']))
-            node.save()
+    # NOTE(TheJulia): A kickstart template is entirely independent
+    # of the stage2 ramdisk. In the end, it was the configuration which
+    # told anaconda how to execute.
+    if i_info.get('ks_template'):
+        # If the value is set, we always overwrite it, in the event
+        # a rebuild is occuring or something along those lines.
+        node.set_driver_internal_info('ks_template',
+                                      i_info['ks_template'])
+    else:
+        _get_image_properties()
+        # ks_template is an optional property on the image
+        if image_properties and 'ks_template' in image_properties:
+            node.set_driver_internal_info(
+                'ks_template', str(image_properties['ks_template']))
+        else:
+            # If not defined, default to the overall system default
+            # kickstart template, as opposed to a user supplied
+            # template.
+            node.set_driver_internal_info(
+                'ks_template',
+                'file://' + os.path.abspath(
+                    CONF.anaconda.default_ks_template
+                )
+            )
+
+    node.save()
 
     for label in labels + anaconda_labels:
         image_info[label] = (
@@ -793,6 +832,7 @@ def build_deploy_pxe_options(task, pxe_info, mode='deploy',
 def build_instance_pxe_options(task, pxe_info, ipxe_enabled=False):
     pxe_opts = {}
     node = task.node
+    isap = node.driver_internal_info.get('is_source_a_path')
 
     for label, option in (('kernel', 'aki_path'),
                           ('ramdisk', 'ari_path'),
@@ -815,6 +855,16 @@ def build_instance_pxe_options(task, pxe_info, ipxe_enabled=False):
                 pxe_opts[option] = os.path.relpath(pxe_info[label][1],
                                                    CONF.pxe.tftp_root)
 
+    # NOTE(TheJulia): This is basically anaconda specific, but who knows
+    # one day! Copy image_source to repo_url if it is a URL to a directory
+    # path, and an explicit stage2 URL is not defined as .treeinfo is totally
+    # a thing and anaconda's dracut element knows the secrets of how to
+    # get and use the treeinfo file. And yes, this is a hidden file. :\
+    # example:
+    # http://mirror.stream.centos.org/9-stream/BaseOS/x86_64/os/.treeinfo
+    if isap and 'stage2_url' not in pxe_opts:
+        pxe_opts['repo_url'] = node.instance_info.get('image_source')
+
     pxe_opts.setdefault('aki_path', 'no_kernel')
     pxe_opts.setdefault('ari_path', 'no_ramdisk')
 
@@ -938,9 +988,10 @@ def build_service_pxe_config(task, instance_image_info,
         pxe_config_path, root_uuid_or_disk_id,
         boot_mode_utils.get_boot_mode(node),
         is_whole_disk_image,
-        deploy_utils.is_trusted_boot_requested(node),
-        deploy_utils.is_iscsi_boot(task), ramdisk_boot,
-        ipxe_enabled=ipxe_enabled, anaconda_boot=anaconda_boot)
+        iscsi_boot=deploy_utils.is_iscsi_boot(task),
+        ramdisk_boot=ramdisk_boot,
+        ipxe_enabled=ipxe_enabled,
+        anaconda_boot=anaconda_boot)
 
 
 def build_kickstart_config_options(task):
@@ -1048,29 +1099,6 @@ def get_volume_pxe_options(task):
     return pxe_options
 
 
-def validate_boot_parameters_for_trusted_boot(node):
-    """Check if boot parameters are valid for trusted boot."""
-    boot_mode = boot_mode_utils.get_boot_mode(node)
-    boot_option = deploy_utils.get_boot_option(node)
-    is_whole_disk_image = node.driver_internal_info.get('is_whole_disk_image')
-    # 'is_whole_disk_image' is not supported by trusted boot, because there is
-    # no Kernel/Ramdisk to measure at all.
-    if (boot_mode != 'bios'
-        or is_whole_disk_image
-        or boot_option != 'netboot'):
-        msg = (_("Trusted boot is only supported in BIOS boot mode with "
-                 "netboot and without whole_disk_image, but Node "
-                 "%(node_uuid)s was configured with boot_mode: %(boot_mode)s, "
-                 "boot_option: %(boot_option)s, is_whole_disk_image: "
-                 "%(is_whole_disk_image)s: at least one of them is wrong, and "
-                 "this can be caused by enable secure boot.") %
-               {'node_uuid': node.uuid, 'boot_mode': boot_mode,
-                'boot_option': boot_option,
-                'is_whole_disk_image': is_whole_disk_image})
-        LOG.error(msg)
-        raise exception.InvalidParameterValue(msg)
-
-
 def validate_kickstart_template(ks_template):
     """Validate the kickstart template
 
@@ -1232,6 +1260,8 @@ def cache_ramdisk_kernel(task, pxe_info, ipxe_enabled=False):
                                              CONF.deploy.http_root,
                                              'stage2')
         ensure_tree(os.path.dirname(file_path))
+
+    if 'ks_cfg' in pxe_info:
         # ks_cfg is rendered later by the driver using ks_template. It cannot
         # be fetched and cached.
         t_pxe_info.pop('ks_cfg')
